@@ -110,6 +110,9 @@ class MobileAppSensor(MobileAppEntity, RestoreSensor):
                 SensorDeviceClass.DATE,
                 SensorDeviceClass.TIMESTAMP,
             )
+            # Only parse strings: if the sensor's state is restored, the state is a
+            # native date or datetime, not str
+            and isinstance(state, str)
             and (timestamp := dt_util.parse_datetime(state)) is not None
         ):
             if self.device_class == SensorDeviceClass.DATE:
