@@ -2,13 +2,14 @@
 from __future__ import annotations
 
 import asyncio
+from dataclasses import dataclass
 from datetime import timedelta
 from functools import partial
-from typing import Any, cast
+from typing import Any
 
 from regenmaschine import Client
 from regenmaschine.controller import Controller
-from regenmaschine.errors import RainMachineError
+from regenmaschine.errors import RainMachineError, UnknownAPICallError
 import voluptuous as vol
 
 from homeassistant.config_entries import ConfigEntry, ConfigEntryState
@@ -28,19 +29,16 @@ from homeassistant.helpers import (
     device_registry as dr,
     entity_registry as er,
 )
-from homeassistant.helpers.entity import DeviceInfo, EntityDescription
-from homeassistant.helpers.update_coordinator import (
-    CoordinatorEntity,
-    DataUpdateCoordinator,
-    UpdateFailed,
-)
+from homeassistant.helpers.entity import DeviceInfo
+from homeassistant.helpers.update_coordinator import CoordinatorEntity, UpdateFailed
+from homeassistant.util.dt import as_timestamp, utcnow
 from homeassistant.util.network import is_ip_address
 
 from .config_flow import get_client_controller
 from .const import (
     CONF_ZONE_RUN_TIME,
-    DATA_CONTROLLER,
-    DATA_COORDINATOR,
+    DATA_API_VERSIONS,
+    DATA_MACHINE_FIRMWARE_UPDATE_STATUS,
     DATA_PROGRAMS,
     DATA_PROVISION_SETTINGS,
     DATA_RESTRICTIONS_CURRENT,
@@ -49,20 +47,20 @@ from .const import (
     DOMAIN,
     LOGGER,
 )
+from .model import RainMachineEntityDescription
+from .util import RainMachineDataUpdateCoordinator
 
 DEFAULT_SSL = True
 
 CONFIG_SCHEMA = cv.removed(DOMAIN, raise_if_present=False)
 
-PLATFORMS = [Platform.BINARY_SENSOR, Platform.SENSOR, Platform.SWITCH]
-
-UPDATE_INTERVALS = {
-    DATA_PROVISION_SETTINGS: timedelta(minutes=1),
-    DATA_PROGRAMS: timedelta(seconds=30),
-    DATA_RESTRICTIONS_CURRENT: timedelta(minutes=1),
-    DATA_RESTRICTIONS_UNIVERSAL: timedelta(minutes=1),
-    DATA_ZONES: timedelta(seconds=15),
-}
+PLATFORMS = [
+    Platform.BINARY_SENSOR,
+    Platform.BUTTON,
+    Platform.SENSOR,
+    Platform.SWITCH,
+    Platform.UPDATE,
+]
 
 CONF_CONDITION = "condition"
 CONF_DEWPOINT = "dewpoint"
@@ -134,6 +132,24 @@ SERVICE_RESTRICT_WATERING_SCHEMA = SERVICE_SCHEMA.extend(
     }
 )
 
+COORDINATOR_UPDATE_INTERVAL_MAP = {
+    DATA_API_VERSIONS: timedelta(minutes=1),
+    DATA_MACHINE_FIRMWARE_UPDATE_STATUS: timedelta(seconds=15),
+    DATA_PROGRAMS: timedelta(seconds=30),
+    DATA_PROVISION_SETTINGS: timedelta(minutes=1),
+    DATA_RESTRICTIONS_CURRENT: timedelta(minutes=1),
+    DATA_RESTRICTIONS_UNIVERSAL: timedelta(minutes=1),
+    DATA_ZONES: timedelta(seconds=15),
+}
+
+
+@dataclass
+class RainMachineData:
+    """Define an object to be stored in `hass.data`."""
+
+    controller: Controller
+    coordinators: dict[str, RainMachineDataUpdateCoordinator]
+
 
 @callback
 def async_get_controller_for_service_call(
@@ -143,12 +159,15 @@ def async_get_controller_for_service_call(
     device_id = call.data[CONF_DEVICE_ID]
     device_registry = dr.async_get(hass)
 
-    if device_entry := device_registry.async_get(device_id):
-        for entry in hass.config_entries.async_entries(DOMAIN):
-            if entry.entry_id in device_entry.config_entries:
-                return cast(
-                    Controller, hass.data[DOMAIN][entry.entry_id][DATA_CONTROLLER]
-                )
+    if (device_entry := device_registry.async_get(device_id)) is None:
+        raise ValueError(f"Invalid RainMachine device ID: {device_id}")
+
+    for entry_id in device_entry.config_entries:
+        if (entry := hass.config_entries.async_get_entry(entry_id)) is None:
+            continue
+        if entry.domain == DOMAIN:
+            data: RainMachineData = hass.data[DOMAIN][entry_id]
+            return data.controller
 
     raise ValueError(f"No controller for device ID: {device_id}")
 
@@ -161,19 +180,19 @@ async def async_update_programs_and_zones(
     Program and zone updates always go together because of how linked they are:
     programs affect zones and certain combinations of zones affect programs.
     """
+    data: RainMachineData = hass.data[DOMAIN][entry.entry_id]
+
     await asyncio.gather(
         *[
-            hass.data[DOMAIN][entry.entry_id][DATA_COORDINATOR][
-                DATA_PROGRAMS
-            ].async_refresh(),
-            hass.data[DOMAIN][entry.entry_id][DATA_COORDINATOR][
-                DATA_ZONES
-            ].async_refresh(),
+            data.coordinators[DATA_PROGRAMS].async_refresh(),
+            data.coordinators[DATA_ZONES].async_refresh(),
         ]
     )
 
 
-async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
+async def async_setup_entry(  # noqa: C901
+    hass: HomeAssistant, entry: ConfigEntry
+) -> bool:
     """Set up RainMachine as config entry."""
     websession = aiohttp_client.async_get_clientsession(hass)
     client = Client(session=websession)
@@ -213,7 +232,11 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
         data: dict = {}
 
         try:
-            if api_category == DATA_PROGRAMS:
+            if api_category == DATA_API_VERSIONS:
+                data = await controller.api.versions()
+            elif api_category == DATA_MACHINE_FIRMWARE_UPDATE_STATUS:
+                data = await controller.machine.get_firmware_update_status()
+            elif api_category == DATA_PROGRAMS:
                 data = await controller.programs.all(include_inactive=True)
             elif api_category == DATA_PROVISION_SETTINGS:
                 data = await controller.provisioning.settings()
@@ -223,37 +246,43 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
                 data = await controller.restrictions.universal()
             else:
                 data = await controller.zones.all(details=True, include_inactive=True)
+        except UnknownAPICallError:
+            LOGGER.info(
+                "Skipping unsupported API call for controller %s: %s",
+                controller.name,
+                api_category,
+            )
         except RainMachineError as err:
             raise UpdateFailed(err) from err
 
         return data
 
+    async def async_init_coordinator(
+        coordinator: RainMachineDataUpdateCoordinator,
+    ) -> None:
+        """Initialize a RainMachineDataUpdateCoordinator."""
+        await coordinator.async_initialize()
+        await coordinator.async_config_entry_first_refresh()
+
     controller_init_tasks = []
     coordinators = {}
-
-    for api_category in (
-        DATA_PROGRAMS,
-        DATA_PROVISION_SETTINGS,
-        DATA_RESTRICTIONS_CURRENT,
-        DATA_RESTRICTIONS_UNIVERSAL,
-        DATA_ZONES,
-    ):
-        coordinator = coordinators[api_category] = DataUpdateCoordinator(
+    for api_category, update_interval in COORDINATOR_UPDATE_INTERVAL_MAP.items():
+        coordinator = coordinators[api_category] = RainMachineDataUpdateCoordinator(
             hass,
-            LOGGER,
+            entry=entry,
             name=f'{controller.name} ("{api_category}")',
-            update_interval=UPDATE_INTERVALS[api_category],
+            api_category=api_category,
+            update_interval=update_interval,
             update_method=partial(async_update, api_category),
         )
-        controller_init_tasks.append(coordinator.async_refresh())
+        controller_init_tasks.append(async_init_coordinator(coordinator))
 
     await asyncio.gather(*controller_init_tasks)
 
     hass.data.setdefault(DOMAIN, {})
-    hass.data[DOMAIN][entry.entry_id] = {
-        DATA_CONTROLLER: controller,
-        DATA_COORDINATOR: coordinators,
-    }
+    hass.data[DOMAIN][entry.entry_id] = RainMachineData(
+        controller=controller, coordinators=coordinators
+    )
 
     await hass.config_entries.async_forward_entry_setups(entry, PLATFORMS)
 
@@ -283,7 +312,13 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     async def async_restrict_watering(call: ServiceCall) -> None:
         """Restrict watering for a time period."""
         controller = async_get_controller_for_service_call(hass, call)
-        await controller.restrictions.restrict(call.data[CONF_DURATION])
+        duration = call.data[CONF_DURATION]
+        await controller.restrictions.set_universal(
+            {
+                "rainDelayStartTime": round(as_timestamp(utcnow())),
+                "rainDelayDuration": duration.total_seconds(),
+            },
+        )
         await async_update_programs_and_zones(hass, entry)
 
     async def async_stop_all(call: ServiceCall) -> None:
@@ -301,7 +336,12 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     async def async_unrestrict_watering(call: ServiceCall) -> None:
         """Unrestrict watering."""
         controller = async_get_controller_for_service_call(hass, call)
-        await controller.restrictions.unrestrict()
+        await controller.restrictions.set_universal(
+            {
+                "rainDelayStartTime": round(as_timestamp(utcnow())),
+                "rainDelayDuration": 0,
+            },
+        )
         await async_update_programs_and_zones(hass, entry)
 
     for service_name, schema, method in (
@@ -406,29 +446,37 @@ class RainMachineEntity(CoordinatorEntity):
     def __init__(
         self,
         entry: ConfigEntry,
-        coordinator: DataUpdateCoordinator,
-        controller: Controller,
-        description: EntityDescription,
+        data: RainMachineData,
+        description: RainMachineEntityDescription,
     ) -> None:
         """Initialize."""
-        super().__init__(coordinator)
+        super().__init__(data.coordinators[description.api_category])
 
-        self._attr_device_info = DeviceInfo(
-            identifiers={(DOMAIN, controller.mac)},
-            configuration_url=f"https://{entry.data[CONF_IP_ADDRESS]}:{entry.data[CONF_PORT]}",
-            connections={(dr.CONNECTION_NETWORK_MAC, controller.mac)},
-            name=str(controller.name).capitalize(),
+        self._attr_extra_state_attributes = {}
+        self._attr_unique_id = f"{data.controller.mac}_{description.key}"
+        self._entry = entry
+        self._data = data
+        self._version_coordinator = data.coordinators[DATA_API_VERSIONS]
+        self.entity_description = description
+
+    @property
+    def device_info(self) -> DeviceInfo:
+        """Return device information about this controller."""
+        return DeviceInfo(
+            identifiers={(DOMAIN, self._data.controller.mac)},
+            configuration_url=(
+                f"https://{self._entry.data[CONF_IP_ADDRESS]}:"
+                f"{self._entry.data[CONF_PORT]}"
+            ),
+            connections={(dr.CONNECTION_NETWORK_MAC, self._data.controller.mac)},
+            name=str(self._data.controller.name).capitalize(),
             manufacturer="RainMachine",
             model=(
-                f"Version {controller.hardware_version} "
-                f"(API: {controller.api_version})"
+                f"Version {self._version_coordinator.data['hwVer']} "
+                f"(API: {self._version_coordinator.data['apiVer']})"
             ),
-            sw_version=controller.software_version,
+            sw_version=self._version_coordinator.data["swVer"],
         )
-        self._attr_extra_state_attributes = {}
-        self._attr_unique_id = f"{controller.mac}_{description.key}"
-        self._controller = controller
-        self.entity_description = description
 
     @callback
     def _handle_coordinator_update(self) -> None:
@@ -439,9 +487,13 @@ class RainMachineEntity(CoordinatorEntity):
     async def async_added_to_hass(self) -> None:
         """When entity is added to hass."""
         await super().async_added_to_hass()
+        self.async_on_remove(
+            self._version_coordinator.async_add_listener(
+                self._handle_coordinator_update, self.coordinator_context
+            )
+        )
         self.update_from_latest_data()
 
     @callback
     def update_from_latest_data(self) -> None:
         """Update the state."""
-        raise NotImplementedError
