@@ -137,6 +137,9 @@ class Thermostat(ZhaEntity, ClimateEntity):
     DEFAULT_MAX_TEMP = 35
     DEFAULT_MIN_TEMP = 7
 
+    _attr_precision = PRECISION_TENTHS
+    _attr_temperature_unit = TEMP_CELSIUS
+
     def __init__(self, unique_id, zha_device, channels, **kwargs):
         """Initialize ZHA Thermostat instance."""
         super().__init__(unique_id, zha_device, channels, **kwargs)
@@ -262,11 +265,6 @@ class Thermostat(ZhaEntity, ClimateEntity):
         """Return the list of available HVAC operation modes."""
         return SEQ_OF_OPERATION.get(self._thrm.ctrl_sequence_of_oper, [HVACMode.OFF])
 
-    @property
-    def precision(self):
-        """Return the precision of the system."""
-        return PRECISION_TENTHS
-
     @property
     def preset_mode(self) -> str:
         """Return current preset mode."""
@@ -334,11 +332,6 @@ class Thermostat(ZhaEntity, ClimateEntity):
             return temp
         return round(temp / ZCL_TEMP, 1)
 
-    @property
-    def temperature_unit(self):
-        """Return the unit of measurement used by the platform."""
-        return TEMP_CELSIUS
-
     @property
     def max_temp(self) -> float:
         """Return the maximum temperature."""
